@@ -5,6 +5,7 @@ use namespace::sweep;
 use Mouse v2.4.5;
 
 use Module::Load ();
+use Aniki::Filter;
 use Aniki::Handler;
 use Aniki::Row;
 use Aniki::Result::Collection;
@@ -12,10 +13,11 @@ use Aniki::Schema;
 use Aniki::QueryBuilder;
 use Aniki::QueryBuilder::Canonical;
 
-our $VERSION = '0.87';
+our $VERSION = '0.88';
 
 use SQL::Maker::SQLType qw/sql_type/;
 use DBIx::Handler;
+use Class::Inspector;
 use Carp qw/croak confess/;
 use Try::Tiny;
 use Scalar::Util qw/blessed/;
@@ -87,7 +89,7 @@ sub setup {
 
     # schema
     if (my $schema_class = $args{schema}) {
-        Module::Load::load($schema_class);
+        Module::Load::load($schema_class) unless Class::Inspector->loaded($schema_class);
 
         my $schema = Aniki::Schema->new(schema_class => $schema_class);
         $class->meta->add_method(schema => sub { $schema });
@@ -98,7 +100,7 @@ sub setup {
 
     # filter
     if (my $filter_class = $args{filter}) {
-        Module::Load::load($filter_class);
+        Module::Load::load($filter_class) unless Class::Inspector->loaded($filter_class);
 
         my $filter = $filter_class->instance();
         $class->meta->add_method(filter => sub { $filter });
@@ -110,7 +112,7 @@ sub setup {
 
     # handler
     if (my $handler_class = $args{handler}) {
-        Module::Load::load($handler_class);
+        Module::Load::load($handler_class) unless Class::Inspector->loaded($handler_class);
         $class->meta->add_method(handler_class => sub { $handler_class });
     }
 
@@ -125,7 +127,7 @@ sub setup {
     {
         my $query_builder_class = $class->use_prepare_cached ? 'Aniki::QueryBuilder::Canonical' : 'Aniki::QueryBuilder';
         if ($args{query_builder}) {
-            Module::Load::load($args{query_builder});
+            Module::Load::load($args{query_builder}) unless Class::Inspector->loaded($args{query_builder});
             $query_builder_class = $args{query_builder};
         }
         my $driver        = $class->_database2driver($class->schema->database);
@@ -138,7 +140,7 @@ sub setup {
         my $root_row_class = 'Aniki::Row';
         my $guess_row_class = sub { $root_row_class };
         if ($args{row}) {
-            Module::Load::load($args{row});
+            Module::Load::load($args{row}) unless Class::Inspector->loaded($args{row});
             $root_row_class = $args{row};
 
             my %table_row_class;
@@ -163,7 +165,7 @@ sub setup {
         my $root_result_class = 'Aniki::Result::Collection';
         my $guess_result_class = sub { $root_result_class };
         if ($args{result}) {
-            Module::Load::load($args{result});
+            Module::Load::load($args{result}) unless Class::Inspector->loaded($args{result});
             $root_result_class = $args{result};
 
             my %table_result_class;
@@ -199,7 +201,14 @@ sub preload_all_result_classes {
     }
 }
 
-sub dbh { shift->handler->dbh }
+sub dbh {
+    my $self = shift;
+    # (for mysql)
+    # XXX: `DBIx::Handler#dbh` send a ping to mysql.
+    #      But, It removes `$dbh->{mysql_insertid}`.
+    return $self->{_context} if exists $self->{_context};
+    return $self->handler->dbh;
+}
 
 sub insert {
     my ($self, $table_name, $row, $opt) = @_;
@@ -269,6 +278,7 @@ sub insert_and_fetch_id {
     my $self = shift;
     local $Carp::CarpLevel = $Carp::CarpLevel + 1;
 
+    local $self->{_context} = $self->dbh;
     $self->insert(@_);
     return unless defined wantarray;
 
@@ -284,6 +294,8 @@ sub insert_and_fetch_row {
     my $table = $self->schema->get_table($table_name) or croak "$table_name is not defined in schema.";
 
     local $Carp::CarpLevel = $Carp::CarpLevel + 1;
+    local $self->{_context} = $self->dbh;
+
     $self->insert($table_name, $row_data, @_);
     return unless defined wantarray;
 
@@ -300,6 +312,8 @@ sub insert_and_emulate_row {
     my $table = $self->schema->get_table($table_name) or croak "$table_name is not defined in schema.";
 
     local $Carp::CarpLevel = $Carp::CarpLevel + 1;
+    local $self->{_context} = $self->dbh;
+
     $row = $self->filter_on_insert($table_name, $row) unless $opt->{no_filter};
 
     $self->insert($table_name, $row, { %$opt, no_filter => 1 });
@@ -331,6 +345,10 @@ sub insert_and_emulate_row {
 
 sub insert_on_duplicate {
     my ($self, $table_name, $insert, $update) = @_;
+    if ($self->schema->database ne 'MySQL') {
+        Carp::croak 'Cannot use insert_on_duplicate (unsupported without MySQL)';
+    }
+
     $insert = $self->filter_on_insert($table_name, $insert);
     $update = $self->filter_on_update($table_name, $update);
 
@@ -351,6 +369,9 @@ sub insert_multi {
 
     my @values = map { $self->filter_on_insert($table_name, $_) } @$values;
     if (exists $opts->{update}) {
+        if ($self->schema->database ne 'MySQL') {
+            Carp::croak 'Cannot use insert_multi with update option (unsupported without MySQL)';
+        }
         $opts->{update} = $self->filter_on_update($table_name, $opts->{update});
     }
 
@@ -362,8 +383,16 @@ sub insert_multi {
         }
     }
 
-    my ($sql, @bind) = $self->query_builder->insert_multi($table_name, \@values, $opts);
-    $self->execute($sql, @bind);
+    if ($self->schema->database eq 'MySQL') {
+        my ($sql, @bind) = $self->query_builder->insert_multi($table_name, \@values, $opts);
+        $self->execute($sql, @bind);
+    }
+    else {
+        $self->txn(sub {
+            local $self->{_context} = shift;
+            $self->insert($table_name, $_, $opts) for @values;
+        });
+    }
     return;
 }
 
@@ -602,7 +631,7 @@ sub _guess_table_name {
 
 # --------------------------------------------------
 # last_insert_id
-sub _fetch_last_insert_id_from_mysql { shift->dbh->{mysql_insertid} };
+sub _fetch_last_insert_id_from_mysql { shift->dbh->{mysql_insertid} }
 sub _fetch_last_insert_id_from_pg {
     my ($self, $table_name, $column) = @_;
     my $dbh = $self->dbh;
