@@ -1,5 +1,5 @@
 package t::DB::Row::Author;
-use Mouse;
+use Mouse v2.4.5;
 extends qw/t::DB::Row/;
 
 1;
